<?php
if (class_exists('OF_Merchant_API')) {
  return;
}

require_once(plugin_dir_path(__FILE__) . 'class-of-helpers.php');

class OF_Merchant_API {
  private $endpoint;
  private $auth_api;
  private $decorators;

  public function __construct($endpoint, $auth_api, $decorators) {
    $this->endpoint = $endpoint;
    $this->auth_api = $auth_api;
    $this->decorators = $decorators;
  }

  public function create_transaction($order) {
    $access_token = $this->auth_api->get_access_token();
    if (is_wp_error($access_token)) {
      return new WP_Error(
        'error', "Cannot get access token."
      );
    }

    $txn_trace_id = OF_Helpers::gen_uuid();
    $response = wp_remote_post(
      "{$this->endpoint}/v1/tenants/partners/transactions?txn_trace_id=$txn_trace_id",
      OF_Helpers::apply_decorators(array(
        'headers' => array(
          'Content-Type' => 'application/json',
          'Authorization' => "Bearer {$access_token}",
        ),
        'body' => json_encode($order),
        'method' => 'POST',
        'data_format' => 'body',
        'timeout' => 60,
      ), $this->decorators)
    );

    OF_Helpers::log($response, 'create_transaction.response');

    $httpStatusCode = $response['response']['code'];
    $body = json_decode($response['body']);
    if ($httpStatusCode < 200 || $httpStatusCode > 299) {
      return new WP_Error(
        'error', "Error processing checkout. Please try again. ({$body->error_code} {$body->message})"
      );
    }

    return array(
      'txn_id' => $body->id,
      'payment_redirect_web_url' => $body->payment_redirect_web_url
    );
  }

  public function is_transaction_approved($id) {
    $url = "{$this->endpoint}/v1/tenants/partners/transactions/{$id}";
    $access_token = $this->auth_api->get_access_token();
    if (is_wp_error($access_token)) {
      return new WP_Error(
        'error', "Cannot get access token."
      );
    }

    $response = wp_remote_get($url, OF_Helpers::apply_decorators(array(
      'headers' => array(
        'Content-Type' => 'application/json',
        'Authorization' => "Bearer $access_token",
      ),
      'timeout' => 60,
    ), $this->decorators));

    OF_Helpers::log($response, 'is_transaction_approved.response');

    $httpStatusCode = $response['response']['code'];
    if ($httpStatusCode < 200 || $httpStatusCode > 299) {
      $errorMessage = json_decode($response['body']);
      return new WP_Error(
        'error', "{$errorMessage->message} ($httpStatusCode)"
      );
    }

    $response_body = wp_remote_retrieve_body($response);
    $response_data = json_decode($response_body, true);
    return strcasecmp($response_data['status'], 'approved') == 0;
  }

  public function configure($webhook_url, $webhook_auth_header, $webhook_auth_value, $pg_name, $pg_config) {
    $payload = array(
      'webhook' => array(
        'type' => 'webhook',
        'subscribed_events' => array('*'),
        'config' => array(
          'url' => $webhook_url,
          'authConfig' => array(
            'header' => $webhook_auth_header,
            'value' => $webhook_auth_value
          )
        )
      ),
      'payment_gateway' => array_merge([
        'pg_name' => $pg_name,
      ], $pg_config)
    );
    OF_Helpers::log($payload, 'webhook_setup.request');

    $url = "{$this->endpoint}/v1/tenants/partners/config";

    $access_token = $this->auth_api->get_access_token();
    if (is_wp_error($access_token)) {
      return new WP_Error(
        'error', "Cannot get access token."
      );
    }
    $response = wp_remote_request($url, OF_Helpers::apply_decorators(array(
      'headers' => array(
        'Content-Type' => 'application/json',
        'Authorization' => "Bearer $access_token",
      ),
      'body' => json_encode($payload),
      'method' => 'PUT',
      'data_format' => 'body',
      'timeout' => 60,
    ), $this->decorators));

    OF_Helpers::log($response, 'webhook_setup.response');

    $httpStatusCode = $response['response']['code'];
    if ($httpStatusCode < 200 || $httpStatusCode > 299) {
      $errorMessage = json_decode($response['body']);
      return new WP_Error(
        'error', "{$errorMessage->message} ($httpStatusCode)"
      );
    }

    return json_decode($response['body']);
  }

  public function refund($order_key, $txn_id, $amount, $reason, $previous_refund_id = '') {
    $body = array(
      'amount' => $amount,
      'reason' => $reason
    );

    $idempotency_key = sha1(
      join( '-', [ $order_key, $previous_refund_id ] )
    );
    OF_Helpers::log( $idempotency_key, 'refund.idempotency_key' );

    $url = "{$this->endpoint}/v1/tenants/partners/transactions/{$txn_id}/refunds";
    $access_token = $this->auth_api->get_access_token();
    if (is_wp_error($access_token)) {
      return new WP_Error(
        'error', "Cannot get access token."
      );
    }
    $response = wp_remote_request($url, OF_Helpers::apply_decorators(array(
      'headers' => array(
        'Content-Type' => 'application/json',
        'Authorization' => "Bearer $access_token",
        'Idempotency-Key' => $idempotency_key,
      ),
      'body' => json_encode($body),
      'method' => 'POST',
      'data_format' => 'body',
      'timeout' => 60,
    ), $this->decorators));

    OF_Helpers::log( $response, 'refund.response' );

    $httpStatusCode = $response['response']['code'];
    if ($httpStatusCode < 200 || $httpStatusCode > 299) {
      $errorMessage = json_decode($response['body']);
      return null;
    }

    $response_body = wp_remote_retrieve_body($response);
    $response_data = json_decode($response_body, true);
    return $response_data;
  }

  public function get_metadata() {
    $url = "{$this->endpoint}/v1/tenants/partners/metadata";
    $access_token = $this->auth_api->get_access_token();
    if (is_wp_error($access_token)) {
      return new WP_Error(
        'error', "Cannot get access token."
      );
    }
    $response = wp_remote_get($url, OF_Helpers::apply_decorators(array(
      'headers' => array(
        'Accept' => 'application/json',
        'Authorization' => "Bearer $access_token",
      ),
      'timeout' => 60,
    ), $this->decorators));

    OF_Helpers::log( $response, 'metadata.response' );

    $status_code = wp_remote_retrieve_response_code( $response );
    if ($status_code < 200 || $status_code > 299) {
      $body = wp_remote_retrieve_body( $response );
      $errorMessage = json_decode($body);
      return new WP_Error(
        'error', "{$errorMessage->message} ($status_code)"
      );
    }

    $body = wp_remote_retrieve_body( $response );
    $data = json_decode( $body, true );
    return $data;
  }
}
